"""
Copyright (c) Huawei Technologies Co., Ltd. 2018-2022. All rights reserved.
"""

# NETCONFSND is the parent class of the SND package. The default configuration has been implemented in the parent class.
# To customize configurations, you need to inherit NetconfSND in the SND package and override methods in the parent class.
from aoc.snd.netconfsnd import NetconfSND

# SysoidInfo specifies the device type, vendor, and model corresponding to the SND package. It is used in the getSysoidInfo method.
from aoc.snd.snd_model_pb2.sysoidinfo_pb2 import SysoidInfo

# Import the following classes and constants: ConnectInfos specifies device connection information;
# ProtocolEntity specifies the device connection protocol; HelloEntity specifies Hello packet information;
# DEFAULT_CONNECT and PRIMARY_CONNECTION are constants. They are used in the getConnectInfo method.
from aoc.snd.snd_model_pb2.connectinfo_pb2 import ConnectInfos, ProtocolEntity, DEFAULT_CONNECT, PRIMARY_CONNECTION,  HelloEntity

# Import the SINGLE_CHANNEL and PROTECTED_MODE constants,
# which are required to configure a single channel or active/standby channels for protection
# during configuration of the device connection capability. They are used in the getConnectInfo method.
from aoc.snd.snd_model_pb2.channelInfo_pb2 import SINGLE_CHANNEL, PROTECTED_MODE

# Import the CommonDriverInfo class to set the general-purpose device driver. (used in the getCommonDriverInfo method)
# NetconfDriverInfo sets the NETCONF driver of a device. (used in the getNetconfDriverInfo method)
from aoc.snd.snd_model_pb2.ecsdriver_pb2 import CommonDriverInfo

# Import the devicemgr.py file to obtain data from devices in real time.
from aoc.snd.snd_model_pb2.ecsdriver_pb2 import NetconfDriverInfo


class NE8000M8NETCONF(NetconfSND):
    def getSysoidInfo(self, aoccontext, request=None):
        """
        Register device information.
        :param request: XML packet
        :param aoc_context: dict, contains
        :return: sysoidInfo
        """
        sysoidInfo = SysoidInfo()
        sysoidEntity = sysoidInfo.sysoidEntity.add()
        # Register the sysoid value of the device.
        sysoidEntity.sysoid = "1.3.6.1.4.1.2011.2.360.1.10"
        # Register the device type information.
        sysoidEntity.deviceType = "ROUTER"
        # Register the device model.
        sysoidEntity.deviceModel = "NetEngine 8000 M8"
        # Register the device vendor information.
        sysoidEntity.deviceVendor = "HUAWEI"
        return sysoidInfo

    def getConnectInfo(self, aoccontext, request=None):
        """
        Configure the connection capability of the device.
        :param request: XML packet
        :param aoc_context: dict, contains
        :return: primaryConnectInfo
        """
        self.logger.info('getConnectInfo start.')
        connectInfos = ConnectInfos()
        primaryConnectInfo = connectInfos.connectInfo.add()
        primaryConnectInfo.connectPolicy = DEFAULT_CONNECT
        """
        If read and write channels are not shared, a single read channel is provided, and active and standby write 
        channels are configured for protection, set is_read_share_write to False, readChannel to SINGLE_CHANNEL, 
        and writeChannel to PROTECTED_MODE.
        """
        # SINGLE_CHANNEL indicates whether to configure a single read or write channel.
        primaryConnectInfo.channelInfo.readChannel = SINGLE_CHANNEL
        # If is_read_share_write is set to True, read and write channels are shared.
        primaryConnectInfo.channelInfo.is_read_share_write = True
        # Set the protocol type to NETCONF.
        primaryConnectInfo.protocolEntity.protocolType = ProtocolEntity.netconf
        primaryConnectInfo.protocolEntity.helloEntity.helloType = HelloEntity.extendType
        primaryConnectInfo.connectionPriority = PRIMARY_CONNECTION
        return primaryConnectInfo

    def getCommonDriverInfo(self, aoccontext, request=None):
        """
        Customize the general-purpose driver of the device.
        :param request: XML packet
        :param aoc_context: dict, contains
        :return: common_driver
        """
        common_driver = CommonDriverInfo()
        """
        Set the operation codes that are not supported by the device. After the configuration, 
        the corresponding operations are automatically converted.
        """
        # The value create indicates that the create operation is not supported and needs to be converted to merge.
        # The value delete indicates that the delete operation is not supported and needs to be converted to remove.
        """
        The value create, delete indicates that the create and delete operations are not supported and need to be 
        converted to merge and remove, respectively.
        """
        common_driver.unsupportedOperations = "create,delete"
        common_driver.deleteStrategy = 1
        syncToDel = common_driver.para.add()
        #Specify whether configuration instances of southbound devices can be deleted during data consistency verification.
        syncToDel.key = "sync-to-del-enable"

        # true: supported; false: not supported
        syncToDel.value = "true"
        return common_driver

    def getNetconfDriverInfo(self, aoccontext, request=None):
        """
        Customize the general-purpose driver of the device.
        :param request: XML packet
        :param aoc_context: dict, contains
        :return: common_driver
        """
        netconf_driver = NetconfDriverInfo()

        """
        Set the number of phases in which device configurations are delivered. The value one indicates that 
        configurations are delivered in one phase; the value two indicates that configurations are delivered in two phases.
        """
        netconf_driver.phase = "two"

        """
        Set the device prototype. 
        The value is huawei-v5 for most Huawei device models and is huawei-v8 for some old Huawei device models.
        """
        netconf_driver.classification = "huawei-v5"

        netconf_driver.modelDiff = "match"

        """
        Specify whether packets are delivered in sequential order. 
        The value set indicates that packets are delivered in non-sequential order.
        """
        netconf_driver.testOption = "set"
        return netconf_driver


